%include %{_rpmconfigdir}/macros.python2

# Don't process requires for .py* files, since it massively increases build time
# and we already know that we depend Python -- we're building it!
%define __requires_exclude_from ^%{_libdir}/.*[.]py[oc]?$

%define major 2
%define minor 7
%define fix 17

Name: python%{major}
Version: %{major}.%{minor}.%{fix}
Release: 0
License: Python-2.0
Summary: Python 2 Programming Language
Url: https://www.python.org

BuildRequires: openssl-devel >= 1.1.1
BuildRequires: zlib-devel
BuildRequires: expat-devel
BuildRequires: libffi-devel
BuildRequires: xz-devel
BuildRequires: bzip2-devel
BuildRequires: ncurses-devel
BuildRequires: readline-devel
BuildRequires: sqlite3-devel
BuildRequires: libutil-devel >= 0.6
# if this is available, _locale will link to it
# Is there a benefit?
# BuildRequires: libintl-devel

Source0: https://www.python.org/ftp/python/%{version}/Python-%{version}.tar.xz
Source1: python2-sitecustomize.py

Patch0: python2-configure.patch
Patch1: python2-Lib_distutils_util.py.patch
Patch2: python2-Lib_sysconfig.py.patch
Patch3: python2-Modules__ctypes_libffi_configure.patch
Patch4: Python-2.7.12-aix.patch
Patch5: Python-2.7.12-aixsetup.patch
Patch6: Python-2.7.12-ldshared.patch
Patch7: Python-2.7.12-linkso.patch
Patch8: Python-2.7.12-pthread-stack-size.patch
Patch9: Python-2.7.12-termios.patch
Patch10: Python-2.7.12-test.patch
Patch11: Python-2.7.13-semaphore.patch
Patch12: python2-o_nofollow.patch


%description
Python is a programming language that lets you work more quickly and integrate your systems more effectively.

You can learn to use Python and see almost immediate gains in productivity and lower maintenance costs.


%package devel
Summary: Python 2 development files
Requires: %{name} = %{version}

%description devel
Python is a programming language that lets you work more quickly and integrate your systems more effectively.

You can learn to use Python and see almost immediate gains in productivity and lower maintenance costs.

%prep
%setup -q -n Python-%{version}

%patch0 -p1
%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%patch6 -p1
%patch7 -p1
%patch8 -p1
%patch9 -p1
%patch10 -p1
%patch11 -p1
%patch12 -p1

%build

# NOTE: To build with IPV6 support you need the following PTFs applied:
# 7.3: MF63822
# 7.2: MF63833
# 7.1: MF63834
#
# Or you may hit this error during configure:
#
# Fatal: You must get working getaddrinfo() function.
#        or you can specify "--disable-ipv6".

# Currently SLIC is limited to supporting AT_FDCWD for dirfd on *at APIs :(
# Since Python internal code will use *at APIs when available, and expects them
# to support more than just AT_FDCWD, we have to disable them or you will just
# get lots of EINVAL returns.

%configure \
  CFLAGS="-std=c99 $CFLAGS" \
  CPPFLAGS="-D_XOPEN_SOURCE=700 -D_ALL_SOURCE" \
  LDFLAGS="-Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib" \
  ARFLAGS="-X${OBJECT_MODE} rc" \
  ac_cv_func_clock_settime=no \
  ac_cv_func_posix_fadvise=no \
  ac_cv_func_posix_fallocate=no \
  ac_cv_func_sched_rr_get_interval=no \
  ac_cv_func_sched_setparam=no \
  ac_cv_func_sched_setscheduler=no \
  ac_cv_func_faccessat=no \
  ac_cv_func_fchmodat=no \
  ac_cv_func_fchownat=no \
  ac_cv_func_fstatat=no \
  ac_cv_func_futimesat=no \
  ac_cv_func_linkat=no \
  ac_cv_func_lstat=yes \
  ac_cv_func_mkdirat=no \
  ac_cv_func_mkfifoat=no \
  ac_cv_func_mknodat=no \
  ac_cv_func_openat=no \
  ac_cv_func_readlinkat=no \
  ac_cv_func_renameat=no \
  ac_cv_func_symlinkat=no \
  ac_cv_func_unlinkat=no \
  ac_cv_func_utimensat=no \
  --with-gcc \
  --with-threads \
  --with-system-expat \
  --with-system-ffi \
  --with-ensurepip=no \
  --without-computed-gotos \
  --enable-ipv6 \
  --enable-shared
  #end

# TODO: Ensure -maix32/-maix64 is passed to gcc when used as the linker. This is hacky. Fix it properly
perl -p -i -e "s|ld_so_aix \$(CC)|ld_so_aix \$(CC) -maix${OBJECT_MODE}|" Makefile

perl -p -i -e 's|#! /usr/local/bin/python|#! %{_bindir}/python2|' Lib/cgi.py # Already patched

gmake

%install

gmake install DESTDIR=%{buildroot}

rm %{buildroot}/%{_bindir}/python
rm %{buildroot}/%{_bindir}/python-config
rm %{buildroot}/%{_bindir}/2to3
rm %{buildroot}/%{_bindir}/idle

%define site_packages "%{buildroot}/%{_libdir}/python%{major}.%{minor}/site-packages"

cp %{SOURCE1} %{site_packages}/sitecustomize.py

%files
%defattr(-, qsys, *none)

%{_bindir}/*

%{_libdir}/python%{major}.%{minor}
%{_libdir}/libpython%{major}.%{minor}.so

%{_mandir}/man1/*

%files devel
%defattr(-, qsys, *none)

%{_includedir}/python%{major}.%{minor}
%{_libdir}/pkgconfig/*

%changelog
* Mon Oct 21 2019 Xu Meng <mengxumx@cn.ibm.com> - 2.7.17
- Update to 2.7.17
- Fix CVE-2019-16935
- Fix CVE-2019-9948
- Fix CVE-2019-9947

* Wed Sep 25 2019 Kevin Adler <kadler@us.ibm.com>
- Disable broken O_NOFOLLOW support on IBM i 7.2

* Wed Aug 07 2019 Mark Irish <mirish@ibm.com> - 2.7.16
- Update openssl-dev version to >= 1.1.1

* Fri Mar 15 2019 - Kevin Adler <kadler@us.ibm.com>
- Require libutil 0.6 (libutil.so.2)

* Tue Mar 05 2019 - Kevin Adler <kadler@us.ibm.com>
- Update to 2.7.16

* Mon Jan 14 2019 - Kevin Adler <kadler@us.ibm.com>
- Speed up automatic dependency generation
- Fix invalid autoconf checks before the fact instead of after

* Wed Jan 09 2019 - Kevin Adler <kadler@us.ibm.com>
- Require fixed libutil

* Tue Jul 10 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com>
- Put replacement of /usr/local/bin/python back

* Tue Jul 10 2018 - mengxumx <mengxumx@cn.ibm.com>
- Python build fails if using make -jxx

* Sun Jul 08 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com>
- Added LINKCC=cc

* Fri Jun 29 2018 - mengxumx <mengxumx@cn.ibm.com>
- Upgrade to 2.7.15

* Thu Mar 22 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com>
- Update to 2.7.14

* Sat Mar 03 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com>
- Added patch for named semaphore problem

* Sat Mar 03 2018 - Kevin Adler <kadler@us.ibm.com>
- Add devel package

* Fri Feb 16 2018 - Kevin Adler <kadler@us.ibm.com>
- Build _locale, needed by YUM

* Tue Feb 13 2018 - Kevin Adler <kadler@us.ibm.com>
- Build bzip2 support, which yum requires

* Tue Feb 13 2018 - Kevin Adler <kadler@us.ibm.com>
- Ensure python builds with OpenSSL 1.0.2

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Add missing sitecustomize.py

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Build with expat, libffi, and ncurses

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Add missing sed-gnu build requirement

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Remove unused sitecustomize.py

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Define _POSIX_SOURCE and _ALL_SOURCE to add various POSIX features to Python,
  fixing an issue with yum running in C locale

* Wed Dec 20 2017 - Kevin Adler <kadler@us.ibm.com>
- Add patches from AIX toolbox and ignore locale package

* Wed Nov 22 2017 - Kevin Adler <kadler@us.ibm.com>
- Add python2 package
